﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;

namespace LoggerSample
{
  public class TraceEvent
  {
    #region private
    private TraceSource m_tracesource;

    private void PrivateTrace(TraceEventType type, int id, string message)
    {
      try
      {
        m_tracesource.TraceEvent(type, id, message);
        m_tracesource.Flush();
      }
      catch (IOException) { }
    }
    #endregion
    #region creator
    /// <summary>
    /// Initializes a new instance of the <see cref="TraceEvent"/> class.
    /// </summary>
    /// <param name="sourcename">The sourcename.</param>
    public TraceEvent(string sourcename)
    {
      m_tracesource = new TraceSource(sourcename);
    }
    /// <summary>
    /// Releases unmanaged resources and performs other cleanup operations before the
    /// <see cref="TraceEvent"/> is reclaimed by garbage collection.
    /// </summary>
    ~TraceEvent()
    {
      TraceEventClose();
    }
    #endregion
    #region public
    /// <summary>
    ///Close the local <see cref="TraceSource"/>
    /// </summary>
    public void TraceEventClose()
    {
      try
      {
        m_tracesource.Flush();
        m_tracesource.Close();
      }
      catch (ObjectDisposedException)
      {
      }
    }
    /// <summary>
    /// regural trace message
    /// </summary>
    /// <param name="type">type of message, e.g. Verbose, Error, etc.. please see <see cref="System.Diagnostics.TraceEventType"/></param>
    /// <param name="id">user identifier for the message</param>
    /// <param name="source">source of mesage</param>
    /// <param name="message">message that we want to trace</param>
    public void Trace(TraceEventType type, int id, string source, string message)
    {
      PrivateTrace(type, id, source + ": " + message);
    }
    /// <summary>
    /// verbose trace message
    /// </summary>
    /// <param name="id">user identifier for the message</param>
    /// <param name="source">source of mesage</param>
    /// <param name="message">message that we want to trace</param>
    public void TraceVerbose(int id, string source, string message)
    {
      Trace(TraceEventType.Verbose, id, source, message);
    }
    /// <summary>
    /// information trace message
    /// </summary>
    /// <param name="id">user identifier for the message</param>
    /// <param name="source">source of mesage</param>
    /// <param name="message">message that we want to trace</param>
    public void TraceInformation(int id, string source, string message)
    {
      Trace(TraceEventType.Information, id, source, message);
    }
    /// <summary>
    /// warning trace message
    /// </summary>
    /// <param name="id">user identifier for the message</param>
    /// <param name="source">source of mesage</param>
    /// <param name="message">message that we want to trace</param>
    public void TraceWarning(int id, string source, string message)
    {
      Trace(TraceEventType.Warning, id, source, message);
    }
    #endregion
  }
}
